	subroutine RingWedgeQuirks (ringwidth, avgslope, wedgeratio, iFilter, &
			         xlowfreqslope, maxpowerwedge, smeardir, smearmag)
		
!       Image Quality Measure (IQM) Program                                                                                       
!       Copyright, THE MITRE Corporation 1992 - 2007                                                                                
!       see Data Rights Notice in HELP SUBROUTINE                                                                             

! 	This subroutine computes: 
!		Average Power in Rings
!		Average Power in angular wedges ("wedges")
!		Smear magnitude and direction
!		Slope of 1-D Ring Power curve in 2 different frequency ranges

!	These computations are not DIRECTLY part of IQ computation, but: 
!	Ringpower is used by Weiner subroutine to determine if noise filter should be applied
!	Wedgepower is used by HVS subroutine to determine if pixel banding exists


! RING:
! *** Determines within which frequency band (i.e range of values in the
! *** frequency plane) each power array data point lies and then averages all
! *** the data points in a given band.  All bands (or rings) are centered about
! *** the origin, contiguous but nonoverlapping.  The first band is always
! *** centered about an integer frequency.  The result is a 1-D power spectrum
! *** placed in the 1-dimensional array called ring().  
! *** ++++++++ This is 'HARD-CODED' for 32 rings (1/64 cy/pixelwidth) +++++++++++++
! *** power spectrum power values are in 'power(col,row)' array
! *** Following formulas determine in which band (ring) a data point lies:
! ***     freqband = int[normalized_radius]
! ***     normalized_radius = (radius + offset) / ringwidth
! ***     radius = sqrt(Xfreq^2 + Yfreq^2)
! ***     offset = ringwidth - firstband_lowvalue
! ***     firstband_lowvalue = first_centervalue - (ringwidth/2)
! ***     first_centervalue = int[1 + (ringwidth/2)]
! *** radius = distance of data point from the origin of the frequency plane
! *** ringcount = array used to count the number of points in each frequency band
! ***    Also computed is the average slope of LogPower vs freq over range:
! ***  .05 to .25 cy/pixwidth (bands 5 to 16); and avg slope over bands 1-3.

! WEDGE:
! ***  Also computed in this subroutine is the average power in wedges:
! ***      prefs file defines wedge angular width (wedgewidth) and
! ***	   lowest frequency at which power summation is started (BeginFreq)
! **   Also computed is smear direction and magnitude, using the wedge data.

	USE globaldata
	
	INTEGER ::  freqband, wedgeangle(3)
	REAL :: binmax(360), binmin(360)
	REAL :: tempor(250)
	

        smearmag = 0.	! smear magnitude in pixel units
        smeardir = 0.	! smear direction in degrees (vert=+90deg, horz=0deg)

!     Initialize Following 4 arrays, whose dimensions are set in GLOBALDATA                                                                                                           
        ringtemp = 0.0	! power in given 180 degree ring x 2
        wedgetemp = 0.0	! power in given wedge x 2
        ringcount = 0	! number of points in given 180 degree ring x 2
        wedgecount = 0	! number of points in given wedge x 2	
 	
        numwedges = NINT(180. / wedgewidth) + 1    ! number of unique wedges+1
        binmax(1) = 90. + wedgewidth/2.
        binmin(1) = 90. - wedgewidth/2.
        DO i = 2, numwedges
		binmax(i) = binmax(i-1) - wedgewidth 
		binmin(i) = binmin(i-1) - wedgewidth 
        END DO
	
	
	

      
!   Compute Ring Constants:
!     Compute width of frequency rings used in converting 2-D power spectrum
!     to 1-D power spectrum; setup for 32 rings (32 frequency bands).
!     number of rings is currently set as constant in IQM routine:  numrings = 32
!     WARNING!! if numrings is changed, there are numerous repercussions
!     in degradation detector threshold values and other aspects of program.      	             

!     limcol,limrow,newcol,newrow used to fill-in power values between r=.5
!     and r=.707107; which is used for 1-D pow spec plot only (for smooth curve at Nyquist)
	
	ringwidth = .5 * REAL(min(xlength,ylength)) / REAL(numrings)
        IF(ringwidth < 1.) ringwidth = 1.
	firstcentervalue = int(1. + ringwidth/2.)
	offset = 3. * ringwidth/2. - firstcentervalue
	lx2 = xlength/2
	ly2 = ylength/2
	fxl2 = REAL(xlength**2)
	fyl2 = REAL(ylength**2)
	limcol = int(sqrt(2.) * REAL(lx2))
	limrow = int(sqrt(2.) * REAL(ly2))
	
	
! *********** RING (FREQUENCY BAND) COMPUTATIONS **************    	
    DO i=1,limcol
	  fi2 = i**2 / fxl2
	  newcol = i
	  DO j = limrow,-limrow,-1
	      radialfreq = sqrt(fi2 + j**2 / fyl2)
	      IF(radialfreq > .707107) GO TO 100
	      newrow = j
	      freqband = int((sqrt(REAL(i**2+j**2))+offset)/ringwidth)
	      IF(i > lx2) then
		  newcol=lx2 - (i-lx2-1)
		  newrow = -j
	      ELSEIF(j > ly2) then
		  newrow = -(ly2-(j-ly2-1))
	      ELSEIF (j < -ly2) then
		  newrow = ly2-(-j-ly2-1)
	      ENDIF
	      ringtemp(freqband) = ringtemp(freqband) + 2.*power(newcol,newrow)
	      ringcount(freqband) = ringcount(freqband) + 2

!	Find angle of each power point and add it into correct angular wedge
!	ref.system:  +90degrees (+y axis), 0degrees (x axis), -90degrees (-y axis)
!	so midangle of first wedge is 90 degrees, midangle of middle wedge is 0 degrees
!	midangle of last wedge is -90 degrees (but this wedge power is same as first, +90 wedge)

	     IF(radialfreq >= BEGINFREQ) then
		  angle = atan(REAL(j)/REAL(i)) * 57.29577951  
		  DO jj=1, numwedges
		     IF(angle >= binmin(jj) .and. angle < binmax(jj)) then
		     		k=jj	
				EXIT	! exit DO jj=1,numwedges loop
		     ENDIF
		  END DO
		  wedgetemp(k) = wedgetemp(k) + 2.* power(newcol,newrow)
		  wedgecount(k) = wedgecount(k) + 2
	     ENDIF
100        END DO
     END DO

! Following adds to wedge(1), that part which is in > 90degree region
! (which is not computed in preceding); uses bilateral symmetrism property
	wedgetemp(1) = wedgetemp(1) + wedgetemp(numwedges)
	wedgecount(1) = wedgecount(1) + wedgecount(numwedges)

! Following adds power along vertical axis 	
	DO j=limrow,-limrow,-1
		newrow = j
		radialfreq = abs(REAL(j)/REAL(ylength))
	  	freqband = int((sqrt(REAL(j**2))+offset)/ringwidth)
	  	IF(j > ly2) newrow = -(ly2-(j-ly2-1))
	  	IF(j < -ly2) newrow = ly2-(-j-ly2-1)
	  	ringtemp(freqband) = ringtemp(freqband) + power(0,newrow)
	  	ringcount(freqband) = ringcount(freqband) + 1
		IF(radialfreq >= BEGINFREQ) then
			wedgetemp(1) = wedgetemp(1) + power(0,newrow)
			wedgecount(1) = wedgecount(1) + 1
		ENDIF
	END DO

! compute AVERAGE POWER in each ring	
	maxringnumber = nint(sqrt(2.) * REAL(numrings))
	DO i = 0, maxringnumber
	  IF (ringcount(i) > 0) ringtemp(i) = ringtemp(i) / ringcount(i)
	END DO

!   Compute AVERAGE POWER in each wedge & find max power wedge 
	 xmaxslice = 0.0
	 DO i=1,numwedges-1
	 	IF(wedgecount(i) /= 0) wedgetemp(i) = wedgetemp(i)/ wedgecount(i)
	 	IF(wedgetemp(i) > xmaxslice) then
	   		xmaxslice = wedgetemp(i)
	   		maxpowerwedge = i
		ENDIF
	END DO
	
	
! Note that when iFilter=1 (set in MainCaller), which is the second Call to this subroutine from MainCaller,
! all remaining code (below IF) in this subroutine is skipped	
IF(iFilter == 0) THEN		! first call to this subroutine in MainCaller
	ring = ringtemp		! rings & wedges NOT weighted by noise or HVS filters
	wedge = wedgetemp	! for use by, e.g., Weiner to determine need for noise filter
ELSE
	ringFilter = ringtemp	 ! second call to this subroutine in MainCaller
	wedgeFilter = wedgetemp  ! rings & wedges weighted by HVS filter, scaling, and noise filter (if applied)
	RETURN			 ! for verbose mode spectra output
ENDIF
	
	
	
! ******* FOLLOWING TESTS FOR IMAGE QUIRKS ARE DONE USING UNFILTERED RINGS & WEDGES *******	
	
!  CCD PIXEL BANDING TEST (e.g., dead or erratic CCD detector element in 1-D sensor)                                                                                
       IF (ring(numrings) > (ccdband * ring(numrings-3) ) ) then 
           nyquistring = numrings-3 	! used in Weiner noise filter
           mcode (8) = '8' 
      ELSE
      	   nyquistring = numrings
      ENDIF   	


!  For very small image size, width < 33 pixels, skip some of the image
!  degradation tests (blur,smear,noise) to prevent division by zero
!  in sparsely populated power spectrum rings and wedges.
!  Also skips these tests if entire image (regardless of size) = constant graylevel
	k=0
	DO j=1,maxringnumber
		if(ring(j) > TinySngl) k=1
	END DO
	IF((xlength < 33) .or. (k == 0)) THEN
!		PRINT*, '**** Skipped image degradation tests (image too small or is uniform gray)   ',imgID
	    	wedgeratio = 0.0
	    	avgslope = 0.0
	    	xlowfreqslope = 0.0
	    	ring(numrings) = 2. * TinySngl	
	    	ring(numrings-3) = 2. * TinySngl	
	    	RETURN
	ENDIF


!   Compute average abs(slope) of LogPower vs. freq over ac bands 4 to 16:
!    (r=.05 to r=.25 for r=1/64 ring width, all based on numrings = 32)
	avgslope = SUM(ABS(LOG10(ring(4:15)/ring(5:16)))*64.) / 12.
	
		
!  Compute average abs(slope) of LogPower vs.freq over ac bands 1 to 3:
	xlowfreqslope = SUM(ABS(LOG10(ring(1:2)/ring(2:3)))*64.) / 2.
	
	
	
! IMAGE SMEAR TEST:
!  First find set of 3 adjacent wedges,which are at&around 90 degrees away
!  from maxpowerwedge; then select the lowest power wedge (from the 3) as smear direction wedge.
! Experiments have shown that this approach, starting with maxpowerwedge, is more consistently 
! accurate in identifying smear direction then approach of simply looking for minimum power wedge.
	
	peakwedgedegrees = 90. - (maxpowerwedge-1) * wedgewidth
	IF(peakwedgedegrees > 0.) then
		xminwedgedegrees = peakwedgedegrees - 90.
	ELSEIF(peakwedgedegrees <= 0.) then
		xminwedgedegrees = peakwedgedegrees + 90.
	ENDIF
	DO jj=1, numwedges
	   IF(xminwedgedegrees >= binmin(jj) .and. xminwedgedegrees < binmax(jj)) then
		k=jj	
		EXIT	! exit DO jj=1,numwedges loop
	   ENDIF
	END DO
	wedgeangle(1) = k-1
	IF( k-1 < 1) wedgeangle(1) = numwedges - 1
	wedgeangle(2) = k
	wedgeangle(3) = k+1
	IF( k+1 > numwedges) wedgeangle(3) = 2
	w1 = wedge(wedgeangle(1))
	w2 = wedge(wedgeangle(2))
	w3 = wedge(wedgeangle(3))


! Select minimum power wedge angle from w1, w2, w3, 
! then wedge ratio for smear test: highestwedgepwer/minimumwedgepower
	IF(w1 < w2 .and. w1 < w3) then
		nnn=1
	ELSEIF(w2 < w3 .and. w2 < w1) then
		nnn=2
	ELSE
		nnn=3
	ENDIF
	
	xminslice = wedge(wedgeangle(nnn))
	wedgeratio = xmaxslice / xminslice
	smeardir = 90. - (wedgeangle(nnn) - 1) * wedgewidth
	

!  to skip following smear mag code, insert here:   GO TO 999

	
! ****************************************************************************	
! FOLLOWING CODE IS SIMPLE APPROACH TO COMPUTE SMEAR MAGNITUDE 	
! IT IS NOT REAL SOPHISTICATED & COULD USE MORE DEVELOPMENT (4/99, nbn)

! Find power in each semi-ring in lowestpowerwedge angle only (xminslice wedge).
! then locate first minimum from semiringpower values vs. cy/pixelwidth 'plot',
! this is assumed to be first  null of smear spectrum (MTF).
! (using previously calculated total ringpower (i.e. over all wedge angles) 
! was not found to be sensitive to true smear null location)

IF(wedgeratio > highsmear) then 	
	ringcount = 0
	tempor = 0.0	
! Next apprx. 35 lines (to ----) is basically same as preceding 'ringpower' code
! but substituting tempor() for ring(), deleting wedge computations, and 
! limiting ringpower sums to lowpowerwedge only:
	DO i=1,limcol
	    fi2 = i**2 / fxl2
	    newcol = i
	    DO j = limrow,-limrow,-1
	        radialfreq = sqrt(fi2 + j**2 / fyl2)
	        IF(radialfreq > .707107) GO TO 111
	        newrow = j
	        freqband = int((sqrt(REAL(i**2+j**2))+offset)/ringwidth)
	        IF(i > lx2) then
		    newcol=lx2 - (i-lx2-1)
		    newrow = -j
	        ELSEIF(j > ly2) then
		    newrow = -(ly2-(j-ly2-1))
	        ELSEIF (j < -ly2) then
		    newrow = ly2-(-j-ly2-1)
	        ENDIF
	        angle = atan(REAL(j)/REAL(i)) * 57.29577951 
	IF(angle >= binmin(wedgeangle(nnn)) .and. angle < binmax(wedgeangle(nnn))) then
	           tempor(freqband) = tempor(freqband) + 2.*power(newcol,newrow)
	           ringcount(freqband) = ringcount(freqband) + 2
	        ENDIF		  	     
111        END DO
       END DO
       
       IF(wedgeangle(nnn) == 1) then
       	   DO j=limrow,-limrow,-1
		newrow = j
		radialfreq = abs(REAL(j)/REAL(ylength))
	  	freqband = int((sqrt(REAL(j**2))+offset)/ringwidth)
	  	IF(j > ly2) newrow = -(ly2-(j-ly2-1))
	  	IF(j < -ly2) newrow = ly2-(-j-ly2-1)
	           tempor(freqband) = tempor(freqband) + power(0,newrow)
	  	   ringcount(freqband) = ringcount(freqband) + 1 		
           END DO
	ENDIF
	
	DO i = 1, maxringnumber
	  IF (ringcount(i) > 0) tempor(i) = tempor(i) / ringcount(i)
	END DO
!  --------------------------------------------------------------------     
	     iflag = 0
	     firstnull = 6.4E21
	     DO i = 1,maxringnumber-2	
	   	IF(tempor(i+1) < tempor(i) .and. tempor(i+2) > tempor(i+1)) then
			firstnull = REAL(i+1)
			EXIT	! exit DO i=1,maxringnumber-2 loop
		END IF
  	     END DO
	     smearmag = 64. / firstnull		! in pixel units
END IF

! ****************************************************************************	
	
		
	
999  	RETURN
	END SUBROUTINE RingWedgeQuirks
	