      SUBROUTINE COMPOWER () 
                                                                                                                                  
!       Image Quality Measure (IQM) Program                                                                                       
!       Copyright, THE MITRE Corporation 1992 - 2007                                                                              
!       see Data Rights Notice in HELP SUBROUTINE                                                                             
                                                                                                                                  
!  This subroutine:                                                                                                          
!  Computes power from the complex values output by fft subroutine: 
!	datar()	1-D array which is real part
!	datai() 1-D array which is imaginary part
!  Results are placed in the 2-D power() array.  The first index of power() indicates                                                         
!  x frequency (thus column index), the second index indicates y frequency (thus row index).  
!  Negative x (negative column) frequencies are not computed since the fft output data is 
!  bilaterally symmetric (i.e. it has complex conjugate symmetry).                                                                         
           
		                                                                                                                          
! *****************************************************************************************                                                                                                                                      
! HISTORY: 
!		Started in 1989 with complex-in & complex-out FFT from Numerical Recipes book 
!			(1st edition, pp.451-453) which is only for 'power-of-2' size square images.
!	    Switched to JPL/Krogh's real-in & complex-out FFT in January 1991;
!			because it is twice as fast and uses half the memory, but is still restricted 
!			to 'power-of-2' size square images.
!		Switched to Brenner's FFT in February 2001, which has a real input & complex output
!			subroutine; it works for ANY image width or height; 
!			neither dimension has to be power-of-2, either dimension can be odd number,
!			image can be Rectangle.  Initially, however, have constrained routine to 
!			arbitray size Square image, until rectangle output is further verified.
!			Ref:  Fortran FFT by Norman Brenner, based on a program by Charles Rader (June, 1967)
!			C. M. Rader and N. M. Brenner, "A new principle for fast Fourier transformation," 
!			IEEE Transactions on Acoustics,Speech, and Signal Processing, vol. ASSP-24, 
!			pp. 264-266, June 1976.                                                          

! PURPOSE OF COMPOWER:                                    
!		All 3 of the above FFTs scramble the output, but in different ways.
!		Also, Krogh's divided output Re&Im parts by imagewidth**2, but NumerRecip & Brenner do not.
!		For Krogh FFT, scaling by imagewidth**2 was normalized out to equate to NumerRecip & Brenner. 
!       In all 3 FFTs, The frequencies from the data-scrambled output of the FFT 
!		are rearranged into a straightforward 2D array representing only 
!		the positive X half of the four quadrant fourier spectrum. i.e, create a 2D array 
!		with positive X axis and both negative and positive Y axes and 
!		store the power values in increasing frequency starting with DC at (x=0,y=0).  

!	Example for 8x8 image:
!		  col 0		col 1		col 2		col 3		col 4		
!		  row 4		
!		  row 3
!		  row 2
!		  row 1
!		  row 0
!		  row -1
!		  row -2
!		  row -3
!	Note that row -4 would have same values as row +4 for even number image height	
!	but is not included because it is really first row of next replication spectrum
!
!


	  USE globaldata	  
                                                                                                                                   
      
	  power = 0.0                                                                                                                                                                                                                                    
      
	  
	IF(mod(xlength,2) /= 0) THEN
		ix = 2	! xlength is odd
	ELSE
		ix = 1	! xlength is even
  	ENDIF
	
	
	DO icol = 0, xlength/2 
		k = icol * xlength + 1
		DO irow = 0, -ylength/2, -1
			power(icol,irow) = DATAR(k)**2 + DATAI(k)**2
			k = k+1
		END DO
		k = icol*xlength + xlength/2 + ix
		DO irow = ylength/2, 1, -1
			power(icol,irow) = DATAR(k)**2 + DATAI(k)**2
			k = k+1
		END DO

		! If ylength is even then power values in top row at +ylength/2 
		! are equal to power values in bottom row at  -ylength/2,
		! but -ylength/2 bottom row is really top row in next replication
		! of image's Fourier transform, so do not include;  Then number of rows
		! used = number of rows in image.
		! If ylengthis odd, these top/bottom rows are not equal and bottom row
		! is used; then number of rows = number of rows in image.
		! For ylength even, need to set power in bottom row = 0 because this
		! row is subsequently used in computations(hvs, scaling, wiener subroutines):
		
		IF(mod(ylength,2) == 0) THEN		! ylength is even
			power(icol, -ylength/2) = 0.0
		ENDIF

	END DO

	
! FOR TESTING ONLY:	
! power spectrum computed directly from real & imaginary components of FFT() output
!   OPEN(UTMP, file='DIAGNOSTIC-Computed_Power', action='WRITE')
!   WRITE(UTMP,*) ' '
!   WRITE(UTMP,*) ' '
!   WRITE(UTMP,*) '  row    col      power      ', imgID
!	DO irow = ylength/2, -ylength/2, -1
!		DO icol = 0, xlength/2
!			WRITE(UTMP, fmt='(2I7,3x,E14.7)') irow, icol, power(icol,irow)
!		END DO
!    END DO
	  	  
	  

 END SUBROUTINE COMPOWER  

