      SUBROUTINE scaling (fl, gsd, alt, pixel, az, look, fwdtilt, sidetilt, gamma, reltivepsilon,&
      			  true_range_gsd,acpitch, acroll, acyaw, xmagmin, numpix)                                                     
                                                                                                                                                          
!       Image Quality Measure (IQM) Program                                                                                       
!       Copyright, THE MITRE Corporation 1992 - 2007                                                                                
!       see Data Rights Notice in HELP SUBROUTINE                                                                             
                                                                                                                                  
!       This subroutine:                                                                                                          
! *** Determines directional scale factor in cycles/ground foot                                                                   
                                                                                                                                  
! *** Specific combinations of parameters in AUXDATA file are used by this                                                       
!     subroutine to select proper computational module.                                                     
                                                                                                                                  
! *** AUXDATA input file parameters:                                                                                                                                                                                                 
!     integer: numpix,jsensor,jsensormode                                                                              
!     real*4:  fl,gsd,alt,pixel,az,fwdtilt,sidetilt,look,gamma,reltivepsilon                                                      
!     real*4:  acroll,acpitch,acyaw,xmagmin                                                                                                                                                   
! *** fl       = focal length in inches                                                                                           
! *** gsd      = ground sample distance in feet; computed from altitude, focal                                                    
!                length, and detector pixel width (not from quality rating),                                                      
!                (note: 'gsd' is not modified by xmagmin value).                                                                  
! *** alt      = altitude in feet                                                                                                 
! *** pixel    = detector pixel width or center-to-center spacing, in microns                                                     
!                (note: 'pixel' is not modified by xmagmin value).                                                                
! *** az       = azimuth in degrees (angle between sensor look direction and                                                      
!                aircraft crosstrack direction)                                                                                                                                                                                
! *** fwdtilt  = magnitude of forward tilt angle in degrees                                                                       
! *** sidetilt = magnitude of side tilt angle in degrees                                                                          
! *** look     = magnitude of look angle in degrees (resultant of fwdtilt &                                                       
!                sidetilt); either (look & az) or (fwdtilt & sidetilt) are                                                        
!                input for oblique sensor                                                                      
! *** gamma      = angle between image array x axis (=line formed by array length                                                 
!                for 1-D detector array) and platform crosstrack line,referenced                                                  
!                to quadrant wherein: 0 deg <= gamma <= 90 deg                                                                                                                               
! *** reltivepsilon = sensor dependent input parameter                                                   
!                   = angular FOV of single detector pixel (general sensor)                                                       
! *** lepsilon = computed true slant range (feet) at field angle epsilon                                                          
! *** lb       = computed cross direction apparent slant range (feet) at field                                                    
!                angle epsilon                                                                                                    
! *** xmagmin = magnification or minification value of workstation image as                                                       
!               displayed and stored, to which psdiqm is applied;                                                                 
!               or sensor pixel aggregation.                                                                                      
!            Examples:                                                                                                            
!            xmagmin = 2.0 if original image is stored in 2X magnified form.                                                      
!            xmagmin = 0.25 if original image is stored in 1/4X minified form.                                                    
!            xmagmin = 0.50 if every 2 sensor pixels on a sensor line are                                                         
!                      aggregated together and output from sensor as 1 pixel.                                                     
!            xmagmin = 1.0 if no magnify, no minify, no aggregation.                                                              
!            xmagmin = (xmagmin for magnification) x (xmagmin for aggregation)                                                    
!                      [if both exist]            
! *** flatcut: if sensor altitude is < flatcut feet, flat earth model used
! *** Equations follow Opt.Eng. paper (April, 1992, pp.813-825)                                                                   

! Various checks are done to make sure sensor parameters that cannot be zero, are in fact not zero;
! the corresponding verification code for some sensors is double length, to grandfather-in auxdatafile
! format used for iqm v6.3 and earlier (which did not use jsensormode to distinguish sensor case)
                                              			  
				                                                                                                                  
      USE GLOBALDATA
	  
     
	  DOUBLE PRECISION :: lepsilon, lb, phitau, phichi, phialfepsi, lzero, dcoslook 
      DOUBLE PRECISION :: gee, que, geegee, queque, geeque, x, y, gamazrad 
      DOUBLE PRECISION :: beta, prange, rad, earth, cyperft, depsilon, dfov, damp, rolloffset 
	  DOUBLE PRECISION :: dfl, dgsd, dalt, dpixel, daz, dlook, dfwdtilt, dsidetilt, dgamma
	  DOUBLE PRECISION :: dreltivepsilon, dacpitch, dacroll, dacyaw, dxmagmin, deltaepsilon
	  DOUBLE PRECISION :: distortion, ep, ss, xdist, ep_prime, depress, azimuth, constant
	  REAL :: look
	  INTEGER:: flag 


! possibly better dble precision statements for cross-platform compatibility ?
!	 INTEGER, PARAMETER :: double = selected_real_kind(precision(0.0)+1)	
!	 real (kind=double) :: lepsilon, lb, ...	 
	  
	  
      rad = 1.74532925199433D-002
	  earth = 2.0903D+007       ! spherical earth radius in feet                                                                                                                       
      flatcut = 100.			! for altitude  < flatcut feet, flat earth model used   
	  
	  
	                                                                                                                                      
!     Computes gamma angle as adjusted for aircraft yaw angle and/or                                                              
!     adjusted for rotation of image as displayed on crt (& as file stored)                                                       
!     acyaw = aircraft yaw + display rotation                                                                                     
      gamyaw = abs (abs (gamma) - acyaw) 
      IF (gamyaw > 180.) gamyaw = gamyaw - 180. 
      IF (gamyaw > 90.) gamyaw = 180. - gamyaw 
      dgamma = dble(abs(gamyaw))                                                                                                                         
 	  
	  dfl = dble(fl)
	  dgsd = dble(gsd)
	  dalt = dble(alt)
	  dpixel = dble(pixel)
	  daz = dble(az)
	  dlook = dble(look)
	  dfwdtilt = dble(fwdtilt)
	  dsidetilt = dble(sidetilt)
	  dreltivepsilon = dble(reltivepsilon)
	  dacpitch = dble(acpitch)
	  dacroll = dble(acroll)
	  dacyaw = dble(acyaw)
	  dxmagmin = dble(xmagmin)                                                                                                                                
   
   	  IF(xmagmin < TinySngl) THEN	
			xmagmin = -1.		! this flags the image (back in MainCaller) as not suitable for processing
			RETURN
	  ENDIF
       
		  
		                                                                          
      IF (jsensor == 1) then 				! General VERTICAL or min info OBLIQUE sensor   
                                                                                                       
           IF(jsensormode == 0) THEN	! this is for old style auxdatafile, iqm v6.3 and earlier
		   		IF (gsd > TinySngl) THEN 			! gsd given
                	dgsd = dgsd / dxmagmin 
                	cyperft = 1.D0 / (2.D0 * dgsd) 
                	flag = 1 
                	GOTO 300                                                                  
           		ELSEIF (pixel>TinySngl .and. alt>TinySngl .and. fl>TinySngl) THEN 	! fl, alt, pixel given
                	cyperft = (12700.D0 * dfl * dxmagmin) / (dalt * dpixel) 
                	flag = 1 
                	GOTO 300                                                    
           		ELSEIF (reltivepsilon>TinySngl .and. alt>TinySngl) THEN ! alt, angular FOV of pixel given  
                	cyperft = dxmagmin / (2.D0 * dalt * dsin (dreltivepsilon * .001D0) ) 
                	flag = 1 
                	GOTO 300 
           		ELSE
					xmagmin = -1.
					RETURN
				ENDIF 
           ELSE							! iqm after v6.3 uses jsensormode parameter
		   		IF (jsensormode == 1 .and. gsd > TinySngl) THEN 		! gsd given	
                	dgsd = dgsd / dxmagmin 
                	cyperft = 1.D0 / (2.D0 * dgsd) 
                	flag = 1 
                	GOTO 300                                                                  
           		ELSEIF (jsensormode == 2 .and. pixel>TinySngl .and. alt>TinySngl .and. fl>TinySngl ) THEN 
                	cyperft = (12700.D0 * dfl * dxmagmin) / (dalt * dpixel) 	! fl, alt (slantrange), pixel given
                	flag = 1 
                	GOTO 300                                                    
           		ELSEIF (jsensormode == 3 .and. reltivepsilon > TinySngl .and. alt > TinySngl ) THEN 
                	cyperft = dxmagmin / (2.D0 * dalt * dsin (dreltivepsilon * .001D0) ) 
                	flag = 1 			! alt(slantrange), angular FOV of pixel given  
                	GOTO 300 
           		ELSE
					xmagmin = -1.
					RETURN
				ENDIF 

		   ENDIF
                                                                                                                                  
                                                                                                       
      ELSEIF (jsensor == 2) THEN 	! general OBLIQUE sensor   
          
           
           IF(jsensormode == 0) THEN    ! this path is to grandfather old auxdata format, iqm prior to v6.5
		   		IF(alt>TinySngl .and. fl>TinySngl .and. pixel>TinySngl) THEN
		   			depsilon = dreltivepsilon 	! other parms directly from auxdatafile:
					 				! alt,fl,pixel,gamma,reltivepsilon,azimuth,look,fwdtilt,sidetilt                                                                                                              
           			ELSEIF(gsd > TinySngl .and. reltivepsilon > TinySngl) THEN
		   			dcoslook = dgsd / dreltivepsilon	! see eq.22 of '92 Opt.Eng.paper
					dlook = DACOS(dcoslook) / rad 		! gsd is in-track GSD
					dalt = 20000.D0				! reltivepsilon parm used to store cross-track GSD
					dfl = 12.D0				! only the fl/alt ratio is important, not absolute values
					dpixel = 25400.D0 * dgsd * dfl * dcoslook / dalt		 
		   		ELSE	
					xmagmin = -1.
					RETURN
		   		ENDIF
	   
	   ! main path for oblique sensor, if fwdtilt & sidetilt are given, they are converted to look & azimuth
	   ELSEIF(jsensormode == 2 .and. alt>TinySngl .and. fl>TinySngl .and. pixel>TinySngl) THEN
		   		depsilon = dreltivepsilon 	! other parms directly from auxdatafile:
					 			! alt,fl,pixel,gamma,reltivepsilon,azimuth,look,fwdtilt,sidetilt                                                                                                                
          
          ! intrack & crosstrack GSDs known, backtrack to other parms that will result in given GSDs:
           ELSEIF(jsensormode == 1 .and. gsd > TinySngl .and. reltivepsilon > TinySngl) THEN
		   		dcoslook = dgsd / dreltivepsilon	! see eq.22 of '92 Opt.Eng.paper
				dlook = DACOS(dcoslook) / rad 		! gsd is in-track GSD
				dalt = 20000.D0				! reltivepsilon parm used to store cross-track GSD
				dfl = 12.D0				! only the fl/alt ratio is important, not absolute values
				dpixel = 25400.D0 * dgsd * dfl * dcoslook / dalt		 
		   ELSE	
				xmagmin = -1.
				RETURN
		   ENDIF
			  
			  			                                                                                                                      
                                                                                                          
      ELSEIF (jsensor == 3) then 	! Aerial FILM Camera    
               				! ref: "Digitization of Reconnaissance Film Imagery", N.B.Nill,                                                     
                       			! MITRE Working Note #95B0197R1, 31 October 1995                                                                                                                                                  
					! fl, pixel, alt are dummy names for this sensor:
		   IF(alt<TinySngl .or. fl<TinySngl .or. pixel<TinySngl) THEN
				xmagmin = -1.
				RETURN
		   ENDIF
           GRD = fl 				! Ground Resolved Distance on film (feet)                                                                                
           ppi = pixel 				! digitizer pixels per inch                                                                 
           photoscale = alt 			! cameraAltitude / {focalLength * cosine(lookangle)}    
           Qfactor = (2. * photoscale) / (ppi * GRD * 12.) 
           IF (Qfactor < .25) then 
                xKfactor = 1. / Qfactor 
           ELSEIF (Qfactor > 4.) then 
                xKfactor = 1. 
           ELSE 
                yg = ALOG10 (Qfactor) 
                xg = .23045 - .5 * yg + .1947 * yg * yg 
                xKfactor = 10.**xg 
           ENDIF 
           gsd = xKfactor * Qfactor * GRD / 2. 
           gsd = gsd / xmagmin 
           cyperft = 1.D0 / (2.D0 * dble(gsd)) 
           flag = 1 
           GOTO 300 
      
	  
	  
	ELSEIF (jsensor == 4) THEN 			! Non-Recon General Sensor (4)  
	       	dgsd = 0.5D0 / dxmagmin 		! not a function of scene scale, so a constant pseudo gsd used
           	cyperft = 1.D0 / (2.D0 * dgsd) 		! but may be function of xmagmin (normal xmagmin=1.0)
           	flag = 1                         	! Same as running sensor #1 with GSD=0.5
           	GOTO 300 
 
	  
	  
	  
	ELSEIF (jsensor == 5) THEN 		! Ground-Based Digital Camera  
           flag = 1                                                                                                         
           IF(jsensormode == 0) THEN		! this is old style auxdatafile, before iqm v6.5 
		   IF (gsd > TinySngl) THEN 	! GSD = detectorpixelwidth projected to scene object
                	dgsd = dgsd / dxmagmin 
                	cyperft = 1.D0 / (2.D0 * dgsd)                 
                	GOTO 300                                                                         
           	   ELSEIF (fl>TinySngl .and. alt>TinySngl .and. pixel>TinySngl) THEN 	
                	cyperft = (12700.D0 * dfl * dxmagmin) / (dalt * dpixel) ! focallength, range, detectorpixelwidth given             
                	GOTO 300 
		   ELSE						! not a function of scene scale
		   			cyperft = 1.
					GOTO 300
		   ENDIF
	   ELSE							! starting with iqm v6.3 jsensormode parm is used
		   IF (jsensormode == 1 .and. gsd > TinySngl) THEN
                	dgsd = dgsd / dxmagmin 
                	cyperft = 1.D0 / (2.D0 * dgsd)                 
                	GOTO 300                                                                         
           	   ELSEIF (jsensormode == 2 .and. fl>TinySngl .and. alt>TinySngl .and. pixel>TinySngl) THEN   
                	cyperft = (12700.D0 * dfl * dxmagmin) / (dalt * dpixel)           
                	GOTO 300 
		   ELSEIF(jsensormode == 3) THEN		! not a function of scene scale
		   		cyperft = 1.
				GOTO 300
		   ELSE
				xmagmin = -1.
				RETURN
		   ENDIF
	   ENDIF
		     
		     
	ELSEIF (jsensor == 6) THEN 			   ! placeholder
           	flag = 1                         	  
           	GOTO 300 
		     
		     
		     
! Starting with v7.0 the 3 tac sensors were moved from jsensor=6,7,8 to jsensor=16,17,18
! jsensor=6 to 15 are empty	  	                                                                                                                           
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            
	                                                                                                                                                                                                                                                                                                   
      ELSEIF (jsensor == 16 .and. allsensors == 'Y') then 			! T1
           dlook = 0.0D0 
           dgsd = 0.0D0 
           damp = daz 
           STOP 'Model for Sensor 16 (T1) is not in code'                                                                                                                    
              
			  
			                                                                                                              
      ELSEIF (jsensor == 17 .and. allsensors == 'Y') then 			! T2  
           dlook = 0.0D0 
           dgsd = 0.0D0 
           depress = dsidetilt 
 	   STOP 'Model for Sensor 17 (T2) is not in code'
	  
	  
       ELSEIF (jsensor == 18 .and. allsensors == 'Y') then 			! T3  
           	dlook = 0.0D0 
           	dgsd = 0.0D0 
           	depsilon = 0.0D0 
       	 	STOP 'Model for Sensor 18 (T3) is not in code'
	
	
	ELSEIF(jsensor < 1 .or. (jsensor > 6 .and. jsensor < 16) .or. (jsensor > 15 .and. allsensors /= 'Y')) THEN                                                                                     	  	  	  	  
	  		xmagmin = -1.
			PRINT*, '**** Invalid sensor number for:   ', imgID
			RETURN
    
    
     ENDIF 	! END OF SENSORS-SPECIFIC CALCULATIONS
          
		                                                                                                                         
      dfwdtilt = dabs(dfwdtilt) 
      dsidetilt = dabs(dsidetilt) 
      IF(dsidetilt < .0001D0) dsidetilt = .0001D0
	  dlook = dabs(dlook) 
      flag = 0
		                                                                                                                          
      IF (DABS(dlook - depsilon)  >= 90.D0) then 
           	xmagmin = -1.
		   	PRINT*, '**** Invalid Case: SensorLookAngle + FieldAngle > 90 degrees    ',imgID 
			RETURN
      ENDIF 
                                                                                                                                
! in following, if fwdtilt & sidetilt are given, they are converted to look angle & azimuth angle:
  IF (alt < flatcut) THEN  ! Compute scale from flat earth model 
		IF ((dfwdtilt >=.1D0) .or. (dsidetilt >= .1D0)) THEN  ! oblique image case 
            x = (dalt * DTAN(dsidetilt*rad))**2
            y = (dalt * DTAN(dfwdtilt*rad))**2
            dlook = DASIN(dsqrt((x+y) / (dalt*dalt+x+y))) / rad
	        IF (DABS(dlook - depsilon)  >= 90.D0) THEN 
           		xmagmin = -1.
		   		PRINT*, '**** Invalid Case: SensorLookAngle + FieldAngle > 90 degrees   ',imgID 
				RETURN
      		ENDIF 
	        daz = DATAN(DTAN(dfwdtilt*rad) / DTAN(dsidetilt*rad)) / rad
		ENDIF
        lepsilon = dalt / DCOS((dlook-depsilon)*rad)
  
  ELSE  ! Compute scale from spherical earth model                                                                                                                                                                               
       IF ((dfwdtilt >= .1D0) .or. (dsidetilt >= .1D0) ) then  ! oblique image case 
            phitau = DASIN((earth + dalt) * DSIN(dfwdtilt*rad) / earth) - dfwdtilt*rad     ! Opt.Eng'92 paper see eq.19                                                                                                    
            phichi = DASIN((earth + dalt) * DSIN(dsidetilt*rad) / earth) - dsidetilt*rad   ! eq.20                                                                                                      
            x = DSQRT(1.D0 - DCOS(phitau) **2 * DCOS(phichi) **2) 
            daz = DASIN(DSIN(phitau) / x) / rad 					! eq.21
            lzero = DSQRT(earth**2 + (earth + dalt)**2 - 2.D0 * earth  &		! eq.15
		            * (earth+dalt) * DCOS(phitau) * DCOS(phichi))                                                                            
            dlook = DASIN(earth * x / lzero) / rad 					! eq.16,18
            IF (DABS(dlook - depsilon)  >= 90.D0) then 
           		xmagmin = -1.
		   		PRINT*, '**** Invalid Case: SensorLookAngle + FieldAngle > 90 degrees    ',imgID 
				RETURN
      		ENDIF 
			lepsilon = lzero 
		ENDIF
		IF(((dfwdtilt < .1D0) .and. (dsidetilt <.1D0)) .or. (dabs(depsilon) >.1D0)) THEN
            x = (dlook - depsilon) * rad 
            phialfepsi = DASIN((earth + dalt) * DSIN(x) / earth) - x 
            lepsilon = DSQRT(earth**2 + (earth + dalt)**2 - 2.D0 &
			           * earth * (earth + dalt) * DCOS(phialfepsi))                                                                                        
  		ENDIF
   ENDIF
  
      lb = lepsilon * DCOS(depsilon * rad) / DCOS( (dlook - depsilon) * rad)                                                                                                        
	  IF (lb < lepsilon) then 
           gee = 1.D0 / lepsilon 
           que = 1.D0 / lb 
      ELSE 
           gee = 1.D0 / lb 
           que = 1.D0 / lepsilon 
      ENDIF 
                                                                                                                                  
      constant = 12700.D0 * dfl / dpixel 
      gamazrad = (dgamma - daz) * rad 
      geegee = gee * gee 
      queque = que * que 
      geeque = gee * que 
	
300   DO  i = 0, xlength/2 
           floi = REAL(i) 
           DO  j = -ylength/2, ylength/2 
                IF (flag /= 1) THEN 
                   IF (i == 0) THEN 
                       beta = gamazrad + 90.D0 * rad 
                   ELSE 
                       beta = gamazrad + atan(abs(REAL(j)) / floi) 
                   ENDIF 
                   prange = DSQRT(geegee * DCOS(beta)**2 + queque * DSIN(beta)**2) / geeque                                                                                                              
                   cyperft = constant / prange
		ENDIF
               power(i,j) = power(i,j) * sngl(cyperft) * xmagmin 
          END DO
      END DO 

      IF (fl < TinySngl) THEN 
           true_range_gsd = gsd 
      ELSE 
           true_range_gsd = sngl((lepsilon * dpixel * dxmagmin) / (25400.D0 * dfl)) 
      ENDIF 

!	for oblique image, compute GSD in ellipse minor axis direction  (crosstrack GSD)
!	i.e.,   i=0, j=ylength/2 in above Do loop; but is this always true?? Needs Verification
        IF (flag /= 1) THEN
        	prange = DSQRT(geegee*DCOS(gamazrad+90.D0*rad)**2 + queque*DSIN(gamazrad+90.D0*rad)**2)/geeque                                                                                                               
        	gsdct= sngl( prange / (2.D0*constant))
	ELSE
		gsdct = true_range_gsd
	ENDIF
!     PRINT*, 'true_range, true_GSD, crosstrack_GSD  ', lepsilon, true_range_gsd, gsdct

      
      END SUBROUTINE scaling                          
                                                                                                                                  
                                                                                                                                  
                                                                                                                                  
                                                                                                                                  
                                                                                   
