      SUBROUTINE wiener (noiseratio, noisevar) 
                                                                                                                                  
!       Image Quality Measure (IQM) Program                                                                                       
!       Copyright, THE MITRE Corporation 1992 - 2007                                                                                
!       see Data Rights Notice in HELP SUBROUTINE                                                                             
                                                                                                                                  
!       This subroutine:                                                                                                          
!   Multiplies power by a rotationally symmetric modified Wiener filter                                                        
!   Global input NOISEFLAG applies to all images in processing run:                                                            
!    noiseflag = 0,  noise filter is applied only if noiseratio < detectlevel                                                                                                                                                                       
!    noiseflag = 1,  user defined noise filter (via 5 parameters in PrefsFile) 
!                    also applied iff noiseratio < detectlevel
!    noiseratio = frequency band 10 / nyquistring

!    nyquistring = band 32 = 0.5 cycles per pixelwidth,
!    but if CCD banding exists (due to dead or erratic CCD detector output), 
!    then nyquistring = band 29
!    Above assumes a band is 1/64 cycles per pixel width. 
                   
!   see Optical Engineering Journal paper, 4/92, eq. 10, pg.819 and Table 1, pg.820 
!   for details on following parameters:

!    noisevar = noise variance, is computed by assuming White Noise                                                      
!  Following 6 parms are given values in PrefsFile, but are only 
!  used in program if noiseflag = 1
!    sigmaS2 = scene variance
!    sigmaG2 = variance of imaging system gaussian MTF
!    rapw = reciprocal of average pulse width
!    DenomExp = the exponent in denominator of eq.10, which = 1.5 in paper
!    Kappa1 = constant (K1)
!    Kappa2 = constant (K2)
     
     
      USE globaldata
          
      REAL :: noiseratio, noisevar, K2, K1, num
      
                                                                                                                                       
      
      pi = 3.141592654 
      noisevar = 0.0 
      rxlength = REAL(xlength) 
      rylength = REAL(ylength) 
            
	             		 
      noiseratio = ring (10) / ring (nyquistring) 	! nyquistring defined in RingWedgeQuirks subroutine
      IF (noiseratio >= detectlevel) RETURN 
      
      mcode (6) = '6'
      IF(psntype == 'AC') THEN
	  	xnormalizer = SNGL(acpower)
      ELSEIF(psntype == 'DC') THEN
	  	xnormalizer = SNGL(dcpower)
      ELSEIF(psntype == 'TC') THEN
	  	xnormalizer = SNGL(totpower)
      ELSE
	  	STOP   'Quitting,  psntype in prefsfile must = AC, DC, or TC'
      ENDIF
      noisevar = (ring(nyquistring) * xnormalizer) / ((rxlength * rylength) **2)
  
      IF(noiseflag == 0) THEN          ! program's standard noise filter      
           K1 = 3355. * 2.** ( - 2 * bpp) * noisevar + 51.2 
	   K2 = 1.5
	   A = .9268
	   DenomPower = 1.5
	   gaussMTFvar = .078
      	   scenevar = (.3125 * 2.**bpp) **2 
      ELSEIF (noiseflag == 1) THEN      ! user supplied noise filter parms
	   K1 = kappa1
	   K2 = kappa2
	   A = rapw
	   DenomPower = DenomExp
	   gaussMTFvar = sigmaG2
	   scenevar = sigmaS2       
      ELSE
      	  PRINT*, 'QUITTING,  noiseflag must be set to 0 or 1 in PrefsFile'
	  PRINT*, 'but it is set to: ',noiseflag
	  STOP
      ENDIF 
      
      CCC = 2. * pi * A * scenevar 
      C = noisevar * K1 
      DO  i = 0, xlength / 2 
           frxl = (i / rxlength) **2 
           DO  j = 0, ylength / 2 
                rsquared = frxl + (REAL(j) / rylength) **2 
                num = CCC * exp ( - rsquared / gaussMTFvar) 
                denom = num + C * (A*A + rsquared) **DenomPower 
                filter = (num / denom) **K2 
                power (i, j) = power (i, j) * filter 
                IF (j /= 0) power (i, - j) = power (i, - j) * filter 
           END DO 
      END DO 
      
      END SUBROUTINE wiener  
      
